using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Windows.Forms;

namespace Menu_Menu_Menu
{
    public class DDS
    {
        public Map map;

        public DDS(Map map)
        {
            this.map = map;
        }

        public void ExtractDDS(BitmapInfo inf, byte[] bChunk, string writePath)
        {
            BinaryWriter bw = new BinaryWriter(new FileStream(writePath, FileMode.Create,
                FileAccess.ReadWrite, FileShare.ReadWrite));

            DDS_Header dds_Header = new DDS_Header();
            bChunk = map.bitm.DecodeByteArray(bChunk, inf, 0, 0);

            dds_Header.Generate(ref inf);
            dds_Header.WriteStruct(ref bw);
            bw.Write(bChunk);
            bw.Close();
        }

        public bool InjectDDS(BitmapInfo inf, string readPath, MainMenu menu, int mapSlotIndx)
        {
            try
            {
                BinaryReader br = new BinaryReader(new FileStream(readPath, FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite));
                //---------------
                DDS_Header dds_Header = new DDS_Header();
                dds_Header.ReadStruct(ref br);

                br.BaseStream.Position = 128;
                byte[] bChunk = map.bitm.EncodeByteArray(br.ReadBytes((int)br.BaseStream.Length - 128), inf, 0, 0);
                //---------------
                br.Close();

                int diff = menu.Slots[mapSlotIndx].ImgSize - bChunk.Length;
                if (diff < 0) //injecting img is bigger
                {
                    MessageBox.Show("Invalid. Filesize is bigger than original", "",
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return false;
                }
                else
                {
                    BinaryWriter bw = new BinaryWriter(new FileStream(map.path,
                        FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite));

                    bw.BaseStream.Position = menu.Slots[mapSlotIndx].ImgOffset;
                    bw.Write(bChunk);

                    if (diff > 0) //injecting img is smaller
                    {
                        byte[] blanks = new byte[diff];
                        bw.Write(blanks);
                    }

                    Array.Clear(menu.Slots[mapSlotIndx].ImgBuffer, 0, menu.Slots[mapSlotIndx].ImgBuffer.Length);
                    bChunk.CopyTo(menu.Slots[mapSlotIndx].ImgBuffer, 0);

                    bw.BaseStream.Position = inf.mapOffset;
                    inf.WriteInfo(bw);

                    bw.Close();
                    return true;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return false;
            }
        }

        #region DDS Header

        //The dwFlags member of the modified DDSURFACEDESC2 structure can be set to one 
        //or more of the following values.
        enum DDSEnum : int
        {
            DDSD_CAPS = 0x1,
            DDSD_HEIGHT = 0x2,
            DDSD_WIDTH = 0x4,
            DDSD_PITCH = 0x8,
            DDSD_PIXELFORMAT = 0x1000,
            DDSD_MIPMAPCOUNT = 0x20000,
            DDSD_LINEARSIZE = 0x80000,
            DDSD_DEPTH = 0x800000,

            DDPF_ALPHAPIXELS = 0x1,
            DDPF_FOURCC = 0x4,
            DDPF_RGB = 0x40,

            //'The dwCaps1 member of the DDSCAPS2 structure can be set to one or more of the following values.
            DDSCAPS_COMPLEX = 0x8,
            DDSCAPS_TEXTURE = 0x1000,
            DDSCAPS_MIPMAP = 0x400000,

            //The dwCaps2 member of the DDSCAPS2 structure can be set to one or more of the following values.
            DDSCAPS2_CUBEMAP = 0x200,
            DDSCAPS2_CUBEMAP_POSITIVEX = 0x400,
            DDSCAPS2_CUBEMAP_NEGATIVEX = 0x800,
            DDSCAPS2_CUBEMAP_POSITIVEY = 0x1000,
            DDSCAPS2_CUBEMAP_NEGATIVEY = 0x2000,
            DDSCAPS2_CUBEMAP_POSITIVEZ = 0x4000,
            DDSCAPS2_CUBEMAP_NEGATIVEZ = 0x8000,
            DDSCAPS2_VOLUME = 0x200000
        }

        //---

        public struct DDS_Header
        {
            public string Magic;
            public DDS_SurfaceDesc dds_SurfaceDesc;

            public void ReadStruct(ref BinaryReader br)
            {
                Magic = new string(br.ReadChars(4));
                dds_SurfaceDesc.ReadStruct(ref br);
            }

            public void WriteStruct(ref BinaryWriter br)
            {
                br.Write(Magic.ToCharArray());
                dds_SurfaceDesc.WriteStruct(ref br);
            }

            public void Generate(ref BitmapInfo bi)
            {
                Magic = "DDS ";
                dds_SurfaceDesc.Generate(ref bi);
            }
        }

        //---

        public struct DDS_SurfaceDesc
        {
            public int size_of_structure;
            public int flags;
            public int height;
            public int width;
            public int PitchOrLinearSize;
            public int depth; //Only used for volume textures.
            public int MipMapCount; //Total Number of MipMaps
            public int[] Reserved1;
            public DDS_PixelFormat dds_PixelFormat;
            public DDS_Caps dds_Caps;
            public int Reserved2;

            public void ReadStruct(ref BinaryReader br)
            {
                size_of_structure = br.ReadInt32();
                flags = br.ReadInt32();
                height = br.ReadInt32();
                width = br.ReadInt32();
                PitchOrLinearSize = br.ReadInt32();
                depth = br.ReadInt32();
                MipMapCount = br.ReadInt32();

                Reserved1 = new int[11]; //11 ints long
                for (int x = 0; x < 11; x++)
                    Reserved1[x] = br.ReadInt32();

                dds_PixelFormat.ReadStruct(ref br);
                dds_Caps.ReadStruct(ref br);

                Reserved2 = br.ReadInt32();
            }

            public void WriteStruct(ref BinaryWriter bw)
            {
                bw.Write(size_of_structure);
                bw.Write(flags);
                bw.Write(height);
                bw.Write(width);
                bw.Write(PitchOrLinearSize);
                bw.Write(depth);
                bw.Write(MipMapCount);

                for (int x = 0; x < 11; x++)
                    bw.Write(Reserved1[x]);

                dds_PixelFormat.WriteStruct(ref bw);
                dds_Caps.WriteStruct(ref bw);

                bw.Write(Reserved2);
            }

            // Generate the DDSURFACEDESC2 chunk of the header
            public void Generate(ref BitmapInfo bi)
            {
                Reserved1 = new int[11];

                //'Size of structure. This member must be set to 124.
                size_of_structure = 124;

                //Flags to indicate valid fields. Always include DDSD_CAPS, DDSD_PIXELFORMAT,
                //DDSD_WIDTH, DDSD_HEIGHT and either DDSD_PITCH or DDSD_LINEARSIZE.
                flags = flags + (int)DDSEnum.DDSD_CAPS;
                flags = flags + (int)DDSEnum.DDSD_PIXELFORMAT;
                flags = flags + (int)DDSEnum.DDSD_WIDTH;
                flags = flags + (int)DDSEnum.DDSD_HEIGHT;

                switch (bi.bitmFormat)
                {
                    case BitmapFormat.DXT1:
                    case BitmapFormat.DXT2_3:
                    case BitmapFormat.DXT4_5:
                        flags = flags + (int)DDSEnum.DDSD_LINEARSIZE;
                        break;

                    default:
                        flags = flags + (int)DDSEnum.DDSD_PITCH;
                        break;
                }

                //Height of the main image in pixels
                height = bi.Height;

                //Width of the main image in pixels
                width = bi.Width;

                //For uncompressed formats, this is the number of bytes per scan line (DWORD aligned) for the main
                //image. dwFlags should include DDSD_PITCH in this case. For compressed formats, this is the (total)
                //number of bytes for the main image. dwFlags should be include DDSD_LINEARSIZE in this case.
                int RGBBitCount = 0;
                switch (bi.bitmFormat)
                {
                    case BitmapFormat.R5G6B5:
                    case BitmapFormat.A1R5G5B5:
                    case BitmapFormat.A4R4G4B4:
                        RGBBitCount = 16;
                        break;

                    case BitmapFormat.P8:
                    case BitmapFormat.X8R8G8B8:
                    case BitmapFormat.A8R8G8B8:
                        RGBBitCount = 32;
                        break;
                }

                PitchOrLinearSize = bi.Width * (RGBBitCount / 8);

                //For volume textures, this is the depth of the volume.
                //dwFlags should include DDSD_DEPTH in this case.
                depth = 0; // There are no volume textures (that I know of) in Halo.

                //For items with mipmap levels, this is the total number of levels in the mipmap
                //chain of the main image. dwFlags should include DDSD_MIPMAPCOUNT in this case
                MipMapCount = bi.MipMapCount;
                if (MipMapCount > 0) flags = flags + (int)DDSEnum.DDSD_MIPMAPCOUNT;

                //A 32-byte value that specifies the pixel format structure.
                dds_PixelFormat.Generate(bi);

                //A 16-byte value that specifies the capabilities structure.
                dds_Caps.Generate(ref bi);
            }
        }

        //---

        public struct DDS_PixelFormat
        {
            public int size; //32
            public int Flags; //4
            public string FourCC; //DXT1, DXT2, etc..
            public int RGBBitCount;
            public int RBitMask;
            public int GBitMask;
            public int BBitMask;
            public int RGBAlphaBitMask;

            public void ReadStruct(ref BinaryReader br)
            {
                size = br.ReadInt32(); //32
                Flags = br.ReadInt32(); //4
                FourCC = new string(br.ReadChars(4)); //DXT1, DXT2, etc..
                RGBBitCount = br.ReadInt32();
                RBitMask = br.ReadInt32();
                GBitMask = br.ReadInt32();
                BBitMask = br.ReadInt32();
                RGBAlphaBitMask = br.ReadInt32();
            }

            public void WriteStruct(ref BinaryWriter bw)
            {
                bw.Write(size);
                bw.Write(Flags);
                bw.Write(FourCC.ToCharArray());

                bw.Write(RGBBitCount);
                bw.Write(RBitMask);
                bw.Write(GBitMask);
                bw.Write(BBitMask);
                bw.Write(RGBAlphaBitMask);
            }

            // Generate the DDPIXELFORMAT chunk of the header
            public void Generate(BitmapInfo bi)
            {
                //Size of structure. This member must be set to 32. 
                size = 32;

                //Flags to indicate valid fields. Uncompressed formats will usually use DDPF_RGB to indicate
                //an RGB format, while compressed formats will use DDPF_FOURCC with a four-character code. 
                //This is accomplished in the below structure.

                //This is the four-character code for compressed formats. dwFlags should include DDPF_FOURCC in
                //this case. For DXTn compression, this is set to "DXT1", "DXT2", "DXT3", "DXT4", or "DXT5". 
                Flags = 0;
                switch (bi.bitmFormat)
                {
                    case BitmapFormat.DXT1:
                        FourCC = "DXT1";
                        Flags = Flags + (int)DDSEnum.DDPF_FOURCC;
                        break;
                    case BitmapFormat.DXT2_3:
                        FourCC = "DXT3";
                        Flags = Flags + (int)DDSEnum.DDPF_FOURCC;
                        break;
                    case BitmapFormat.DXT4_5:
                        FourCC = "DXT4";
                        Flags = Flags + (int)DDSEnum.DDPF_FOURCC;
                        break;
                    default:
                        FourCC = "\0\0\0\0";
                        Flags = Flags + (int)DDSEnum.DDPF_RGB;
                        break;
                }

                //For RGB formats, this is the total number of bits in the format. dwFlags should include DDPF_RGB
                //in this case. This value is usually 16, 24, or 32. For A8R8G8B8, this value would be 32. 
                switch (bi.bitmFormat)
                {
                    case BitmapFormat.R5G6B5:
                        RGBBitCount = 16;
                        break;

                    case BitmapFormat.A1R5G5B5:
                    case BitmapFormat.A4R4G4B4:
                        RGBBitCount = 16;
                        Flags = Flags + (int)DDSEnum.DDPF_ALPHAPIXELS;
                        break;

                    case BitmapFormat.X8R8G8B8:
                        //Flags = Flags + (int)DDSEnum.DDPF_ALPHAPIXELS;
                        RGBBitCount = 32;
                        break;

                    case BitmapFormat.P8:
                    case BitmapFormat.A8R8G8B8:
                        Flags = Flags + (int)DDSEnum.DDPF_ALPHAPIXELS;
                        RGBBitCount = 32;
                        break;

                    default:
                        RGBBitCount = 0;
                        break;
                }

                //For RGB formats, this contains the masks for the red, green, and blue channels. For A8R8G8B8, these
                //values would be 0x00ff0000, 0x0000ff00, and 0x000000ff respectively. 
                switch (bi.bitmFormat)
                {
                    case BitmapFormat.R5G6B5:
                        RBitMask = 0xF800;
                        GBitMask = 0x7E0;
                        BBitMask = 0x1F;
                        RGBAlphaBitMask = 0x0;
                        break;

                    case BitmapFormat.A1R5G5B5:
                        RBitMask = 0x7C00;
                        GBitMask = 0x3E0;
                        BBitMask = 0x1F;
                        RGBAlphaBitMask = 0x8000;
                        break;

                    case BitmapFormat.A4R4G4B4:
                        RBitMask = 0xF00;
                        GBitMask = 0xF0;
                        BBitMask = 0xF;
                        RGBAlphaBitMask = 0xF000;
                        break;

                    case BitmapFormat.X8R8G8B8:
                        RBitMask = 0xFF0000;
                        GBitMask = 0xFF00;
                        BBitMask = 0xFF;
                        RGBAlphaBitMask = 0x0;
                        break;

                    case BitmapFormat.P8:
                    case BitmapFormat.A8R8G8B8:
                        RBitMask = 0xFF0000;
                        GBitMask = 0xFF00;
                        BBitMask = 0xFF;
                        RGBAlphaBitMask = int.Parse("FF000000", System.Globalization.NumberStyles.HexNumber);
                        break;

                    default:
                        RBitMask = 0;
                        GBitMask = 0;
                        BBitMask = 0;
                        RGBAlphaBitMask = 0x0;
                        break;
                }

                //For RGB formats, this contains the mask for the alpha channel, if any. dwFlags should include 
                //DDPF_ALPHAPIXELS in this case. For A8R8G8B8, this value would be 0xff000000. 
                //^^ This was set in the previous block of code.
            }
        }

        //---

        public struct DDS_Caps
        {
            public int caps1;
            public int caps2;
            public int[] Reserved;

            public void ReadStruct(ref BinaryReader br)
            {
                caps1 = br.ReadInt32();
                caps2 = br.ReadInt32();
                Reserved = new int[2];
                Reserved[0] = br.ReadInt32();
                Reserved[1] = br.ReadInt32();
            }

            public void WriteStruct(ref BinaryWriter bw)
            {
                bw.Write(caps1);
                bw.Write(caps2);
                bw.Write(0);//Reserved[0]);
                bw.Write(0); //Reserved[1]);
            }

            // Generate the DDSCAPS2 chunk of the header
            public void Generate(ref BitmapInfo b2)
            {
                //DDS files should always include DDSCAPS_TEXTURE. If the file contains mipmaps, DDSCAPS_MIPMAP
                //should be set. For any DDS file with more than one main surface,such as a mipmaps, cubic
                //environment map, or volume texture, DDSCAPS_COMPLEX should also be set. 
                caps1 = caps1 + (int)DDSEnum.DDSCAPS_TEXTURE;
                if (b2.MipMapCount > 0) caps1 = caps1 + (int)DDSEnum.DDSCAPS_MIPMAP;
                caps1 = caps1 + (int)DDSEnum.DDSCAPS_COMPLEX;

                //For cubic environment maps, DDSCAPS2_CUBEMAP should be included as well as one or more faces of
                //the map (DDSCAPS2_CUBEMAP_POSITIVEX, DDSCAPS2_CUBEMAP_NEGATIVEX, DDSCAPS2_CUBEMAP_POSITIVEY,
                // DDSCAPS2_CUBEMAP_NEGATIVEY, DDSCAPS2_CUBEMAP_POSITIVEZ, DDSCAPS2_CUBEMAP_NEGATIVEZ). For volume
                //textures, DDSCAPS2_VOLUME should be included. 
                //******************************************************************************
                //This is where I stopped - this code needs to be added for cubemaps.
                //******************************************************************************
                caps2 = 0;
            }
        }

        #endregion
    }
}
