using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Windows.Forms;

namespace Menu_Menu_Menu
{
    public struct Tag
    {
        public string Type;
        public int Ident;

        public int RawMetaOffset;
        public int MetaOffset;
        public int MetaSize;

        public int PathOffset;
        public string Path;
        public int PathLength;
    }

    public class Tags
    {
        public Map map;
        public Tag[] tagsList;
        public List<string> tagTypes;

        public Tags(Map map, BinaryReader br)
        {
            this.map = map;

            br.BaseStream.Position = map.index.offsetOfObjectIndex;

            //will store the tags of the map
            tagsList = new Tag[map.index.objectCount];
            tagTypes = new List<string>();

            //count through the 'said' tag amount
            for (int x = 0; x < map.index.objectCount; x++)
            {
                //this is the tagtype, they are backward in map so we reverse
                string tt = map.Reverse(new string(br.ReadChars(4)));

                //if its not in the list add it
                if (!tagTypes.Contains(tt)) tagTypes.Add(tt);

                //and this is the rest of the info
                tagsList[x].Type = tt;
                tagsList[x].Ident = br.ReadInt32();
                tagsList[x].RawMetaOffset = br.ReadInt32();
                tagsList[x].MetaOffset = tagsList[x].RawMetaOffset - map.index.secondaryMagic;
                tagsList[x].MetaSize = br.ReadInt32();
            }

            tagTypes.Sort();

            //go to the filetable which contains all the strings - globals/globals etc
            br.BaseStream.Position = (int)map.header.fileTableOffset;

            //go through the tags and add in their paths
            char chr;
            for (int i = 0; i < map.index.objectCount; i++)
            {
                tagsList[i].PathOffset = (int)br.BaseStream.Position;

                while ((chr = br.ReadChar()) != '\0')
                    tagsList[i].Path += chr;

                tagsList[i].PathLength = tagsList[i].Path.Length;
            }
        }

        public int IndexOf(int ID)
        {
            //go through tags
            for (int x = 0; x < tagsList.Length; x++)
            {
                //if theres an ID match
                if (tagsList[x].Ident == ID)
                    return x; //return the tags index in the list
            }
            return -1; //no matches, so return -1
        }

        public int IndexOf(string type, string path)
        {
            //go through tags
            for (int x = 0; x < tagsList.Length; x++)
            {
                //if theres a type and path match 
                if (tagsList[x].Type == type && tagsList[x].Path == path)
                    return x; //return the tags index in the list
            }
            return -1; //no matches, so return -1
        }
    }
}