using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Drawing;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Drawing.Imaging;

namespace Menu_Menu_Menu
{
    public class MainMenu
    {
        #region Instance Data

        public Map Map;
        public bool Loaded;
        public bool Ready;

        public int SlotsOffset;
        public int SlotsCount;
        public MapSlot[] Slots;


        private MapFilter mapFilter;
        public int CurReflexiveInMeta;
        public int CurSlotSize;

        #endregion

        #region Accessors

        public MapFilter MapFilter
        {
            get { return mapFilter; }
            set
            {
                switch (mapFilter = value)
                {
                    case MapFilter.SinglePlayer:
                        CurReflexiveInMeta = 15812;
                        CurSlotSize = 2896;
                        break;

                    case MapFilter.MultiPlayer:
                        CurReflexiveInMeta = 15820;
                        CurSlotSize = 3172;
                        break;
                }
            }
        }

        #endregion

        public MainMenu(Map map)
        {
            this.Map = map;
            this.MapFilter = MapFilter.SinglePlayer;
        }

        #region Load Code

        public void Load()
        {
            BinaryReader br = new BinaryReader(new FileStream(Map.path,
               FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite));

            br.BaseStream.Position = Map.tags.tagsList[0].MetaOffset + CurReflexiveInMeta;

            SlotsCount = br.ReadInt32();
            SlotsOffset = br.ReadInt32() - Map.index.secondaryMagic;

            if (SlotsOffset < 0)
            {
                Loaded = false;
            }
            else
            {
                Slots = new MapSlot[SlotsCount];
                br.BaseStream.Position = SlotsOffset;

                // Get info.
                for (int x = 0; x < SlotsCount; x++)
                {
                    Slots[x] = GetMapSlot(br);
                }
                // Get images.
                for (int x = 0; x < SlotsCount; x++)
                {
                    ReadMapSlotImage(br, x);
                }

                Loaded = true;
            }

            br.Close();
        }

        public MapSlot GetMapSlot(BinaryReader br)
        {
            MapSlot ms = new MapSlot();

            ms.MetaOffset = br.BaseStream.Position;
            if (MapFilter == MapFilter.SinglePlayer) br.BaseStream.Position += 4;

            ms.StartValue = br.ReadInt32();
            ms.bitmTag = br.ReadBytes(4);
            ms.TagID = br.ReadInt32();

            // Diff lang names.
            ms.Names = new string[8];
            for (int x = 0; x < ms.Names.Length; x++)
            {
                ms.Names[x] = Encoding.Unicode.GetString(br.ReadBytes(64)).Split('\0')[0];
            }
            // Internal name.
            ms.internalName = Encoding.Unicode.GetString(br.ReadBytes(64)).Split('\0')[0];

            // Diff lang descriptions.
            ms.Descriptions = new string[8];
            for (int x = 0; x < ms.Descriptions.Length; x++)
            {
                ms.Descriptions[x] = Encoding.Unicode.GetString(br.ReadBytes(256)).Split('\0')[0];
            }
            // Internal description
            ms.internalDescription = Encoding.Unicode.GetString(br.ReadBytes(256)).Split('\0')[0];

            if (MapFilter == MapFilter.MultiPlayer)
            {
                ms.Scenario = Encoding.ASCII.GetString(br.ReadBytes(256)).Split('\0')[0];
                ms.EndValue = br.ReadInt32();
                ms.Locked = br.ReadInt32();
                ms.TeamOptions = br.ReadInt32();

                br.BaseStream.Position += 12;
            }

            return ms;
        }

        public void ReadMapSlotImage(BinaryReader br, int slotIndex)
        {
            int tagIndx = Map.tags.IndexOf(Slots[slotIndex].TagID);
            br.BaseStream.Position = Map.tags.tagsList[tagIndx].MetaOffset + 68 + 4;

            int tr = br.ReadInt32() - Map.index.secondaryMagic;
            br.BaseStream.Position = tr;

            Slots[slotIndex].bInfo = new BitmapInfo(br);
            br.BaseStream.Position += 4;

            // Only get first LOD.
            Slots[slotIndex].ImgOffset = br.ReadInt32();
            br.BaseStream.Position += 20;
            Slots[slotIndex].ImgSizePointer = br.BaseStream.Position;
            Slots[slotIndex].ImgSize = br.ReadInt32();

            br.BaseStream.Position = Slots[slotIndex].ImgOffset;
            Slots[slotIndex].ImgBuffer = br.ReadBytes(Slots[slotIndex].ImgSize);

            Slots[slotIndex].Image = Map.bitm.DecodeByteArrayAndReturnBitmap(
                Slots[slotIndex].ImgBuffer, Slots[slotIndex].bInfo, 0, 0);
        }

        #endregion

        #region Save Code

        public void SaveMapSlot(int slotIndex)
        {
            BinaryWriter bw = new BinaryWriter(new FileStream(Map.path,
              FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite));

            saveSlot(bw, slotIndex);

            bw.Close();
        }

        public void SaveAllChanges()
        {
            BinaryWriter bw = new BinaryWriter(new FileStream(Map.path,
              FileMode.Open, FileAccess.ReadWrite, FileShare.ReadWrite));

            for (int m = 0; m < Slots.Length; m++)
            {
                saveSlot(bw, m);
            }

            bw.Close();
        }

        private void saveSlot(BinaryWriter bw, int slotIndex)
        {
            bw.BaseStream.Position = Slots[slotIndex].MetaOffset;
            if (MapFilter == MapFilter.SinglePlayer) bw.BaseStream.Position += 4;

            bw.Write(Slots[slotIndex].StartValue);
            bw.Write(Slots[slotIndex].bitmTag);
            bw.Write(Slots[slotIndex].TagID);

            Encoding en = Encoding.Unicode;
            byte[] bUnicode;
            int nLen;

            // Diff lang names.
            for (int x = 0; x < Slots[slotIndex].Names.Length; x++)
            {
                bUnicode = Encoding.Unicode.GetBytes(Slots[slotIndex].Names[x]);
                bw.Write(bUnicode);
                nLen = 64 - bUnicode.Length;
                if (nLen > 0) bw.Write(new byte[nLen]);
            }
            // Internal name.
            bUnicode = Encoding.Unicode.GetBytes(Slots[slotIndex].internalName);
            bw.Write(bUnicode);
            nLen = 64 - bUnicode.Length;
            if (nLen > 0) bw.Write(new byte[nLen]);

            // Diff lang descriptions.
            for (int x = 0; x < Slots[slotIndex].Descriptions.Length; x++)
            {
                bUnicode = Encoding.Unicode.GetBytes(Slots[slotIndex].Descriptions[x]);
                bw.Write(bUnicode);
                nLen = 256 - bUnicode.Length;
                if (nLen > 0) bw.Write(new byte[nLen]);
            }

            // Internal description.
            bUnicode = Encoding.Unicode.GetBytes(Slots[slotIndex].internalDescription);
            bw.Write(bUnicode);
            nLen = 256 - bUnicode.Length;
            if (nLen > 0) bw.Write(new byte[nLen]);

            if (MapFilter == MapFilter.MultiPlayer)
            {
                bw.Write(Slots[slotIndex].Scenario.ToCharArray());
                nLen = 256 - Slots[slotIndex].Scenario.Length;
                if (nLen > 0) bw.Write(new byte[nLen]);

                bw.Write(Slots[slotIndex].EndValue);
                bw.Write(Slots[slotIndex].Locked);
                bw.Write(Slots[slotIndex].TeamOptions);
            }

            // Image.
            //bw.BaseStream.Position = Slots[slotIndex].ImgOffset;
            //bw.Write(Slots[slotIndex].ImgBuffer);
        }

        #endregion

        #region Misc

        public void ReadMapFile(string fileName, int slotIndex)
        {
            BinaryReader br = new BinaryReader(new FileStream(fileName, FileMode.Open,
                FileAccess.ReadWrite, FileShare.ReadWrite));

            //map internal
            br.BaseStream.Position = 408;
            string internalName = Encoding.ASCII.GetString(br.ReadBytes(32)).Split('\0')[0];

            //langs
            for (int x = 0; x < Slots[slotIndex].Names.Length; x++)
                Slots[slotIndex].Names[x] = internalName;

            //mm internal
            Slots[slotIndex].internalName = internalName;

            if (MapFilter == MapFilter.MultiPlayer)
            {
                //scenario
                br.BaseStream.Position = 444;
                Slots[slotIndex].Scenario = 
                    Encoding.ASCII.GetString(br.ReadBytes(256)).Split('\0')[0];
            }

            br.Close();
        }

        public void ClearMapSlot(int slotIndex)
        {
            if (MapFilter == MapFilter.MultiPlayer)
                Slots[slotIndex].Locked = 1;

            //langs
            for (int x = 0; x < Slots[slotIndex].Names.Length; x++)
            {
                Slots[slotIndex].Names[x] = string.Empty;
                Slots[slotIndex].Descriptions[x] = string.Empty;
            }

            //internal
            Slots[slotIndex].internalName = string.Empty;

            if (MapFilter == MapFilter.MultiPlayer)
                Slots[slotIndex].Scenario = string.Empty;

            Slots[slotIndex].internalDescription = string.Empty;
        }

        public void CreateNewRaw(int slotIndex)
        {
            FileStream fs = new FileStream(Map.path, FileMode.Open,
                FileAccess.ReadWrite, FileShare.ReadWrite);

            BinaryReader br = new BinaryReader(fs);
            BinaryWriter bw = new BinaryWriter(fs);

            //goto index in Map (stright after bitmaps)
            br.BaseStream.Position = Map.header.indexOffset;

            //store Map downward from pos
            byte[] bottomHalf = br.ReadBytes((int)(br.BaseStream.Length - br.BaseStream.Position));

            //goto first reflexive of bitm tag
            br.BaseStream.Position = Map.tags.tagsList[Map.tags.IndexOf(
                Slots[slotIndex].TagID)].MetaOffset + 68;

            int cc = br.ReadInt32();
            int tr = br.ReadInt32() - Map.index.secondaryMagic;

            //go back to index offset
            bw.BaseStream.Position = Map.header.indexOffset;

            List<long> Pointers = new List<long>();
            List<int> NewOffsets = new List<int>();

            for (int x = 0; x < cc; x++)
            {
                for (int y = 0; y < 6; y++)
                {
                    int newbOffset = (int)bw.BaseStream.Position;

                    br.BaseStream.Position = tr + (x * 116) + 28 + (y * 4);

                    int bOffset = br.ReadInt32();
                    if (bOffset == -1)
                    {
                        bw.BaseStream.Position = newbOffset;
                        break;
                    }

                    Pointers.Add(br.BaseStream.Position - 4);
                    NewOffsets.Add(newbOffset);

                    br.BaseStream.Position += 20;
                    int bSize = br.ReadInt32();

                    //goto bitmap in Map
                    br.BaseStream.Position = bOffset;
                    byte[] bArray = br.ReadBytes(bSize);

                    bw.BaseStream.Position = newbOffset;
                    bw.Write(bArray);

                    //write bitmap padding
                    byte[] padding = new byte[512 - (bSize % 512)];
                    if (padding.Length != 512)
                    {
                        bw.Write(padding);
                    }
                }
            }

            //now we need to shift
            int shift = (int)bw.BaseStream.Position - Map.header.indexOffset;

            Map.header.indexOffset = (int)bw.BaseStream.Position;
            bw.Write(bottomHalf);

            //--[Header]
            bw.BaseStream.Position = 16;
            bw.Write(Map.header.indexOffset);

            //fix offsets
            Map.header.metaStart += shift;

            Map.index.primaryMagic -= shift;
            Map.index.offsetOfObjectIndex += shift;
            Map.index.secondaryMagic -= shift;

            for (int x = 0; x < Pointers.Count; x++)
            {
                bw.BaseStream.Position = (Pointers[x] += shift);
                bw.Write(NewOffsets[x]);
            }

            Slots[slotIndex].ImgOffset = NewOffsets[0];
            for (int x = 0; x < SlotsCount; x++)
            {
                Slots[x].bInfo.mapOffset += shift;
                Slots[x].ImgSizePointer += shift;
                Slots[x].MetaOffset += shift;
            }

            //fix meta offsets
            for (int x = 0; x < Map.tags.tagsList.Length; x++)
            {
                if (Map.tags.tagsList[x].Type != "sbsp" && Map.tags.tagsList[x].Type != "ltmp")
                    Map.tags.tagsList[x].MetaOffset += shift;
            }

            //--[End]
            bw.BaseStream.Position =
                Map.tags.tagsList[Map.tags.tagsList.Length - 1].MetaOffset +
                Map.tags.tagsList[Map.tags.tagsList.Length - 1].MetaSize;

            int newPaddingLen = 512 - ((int)bw.BaseStream.Position % 512);
            newPaddingLen += (newPaddingLen == 512 ? 2560 : 3072);

            int eof = (int)bw.BaseStream.Position + newPaddingLen;

            while (bw.BaseStream.Position < eof)
                bw.Write((byte)205); //CD

            bw.BaseStream.SetLength(eof);

            //--[Header]
            bw.BaseStream.Position = 8;
            bw.Write(eof);

            br.Close();
            bw.Close();
        }

        public void GenStartEndValues(int slotIndex)
        {
            //find highest value
            int highval = 0;
            for (int x = 0; x < Slots.Length; x++)
            {
                if (x == slotIndex) continue;
                if (Slots[x].StartValue > highval) highval = Slots[x].StartValue;
                if (Slots[x].EndValue > highval) highval = Slots[x].EndValue;
            }

            //new val
            highval += 10;

            Slots[slotIndex].StartValue = highval;

            if (MapFilter == MapFilter.MultiPlayer)
                Slots[slotIndex].EndValue = highval;
        }

        public bool RawIsInUseByOthers(int slotIndex)
        {
            for (int x = 0; x < SlotsCount; x++)
            {
                if (x == slotIndex) continue;

                if (Slots[x].ImgOffset == Slots[slotIndex].ImgOffset)
                    return true;
            }
            return false;
        }

        #endregion

        /// <summary>
        /// W.I.P
        /// </summary>
        /// <param name="MapSlotIndx"></param>
        public void DuplicateMapSlot(int slotIndex)
        {
            FileStream fs = new FileStream(Map.path, FileMode.Open,
                FileAccess.ReadWrite, FileShare.ReadWrite);

            BinaryReader br = new BinaryReader(fs);
            BinaryWriter bw = new BinaryWriter(fs);

            //--[Add new slot to matg]
            br.BaseStream.Position = Slots[slotIndex].MetaOffset;
            byte[] dupArray = br.ReadBytes(CurSlotSize);

            br.BaseStream.Position = Slots[Slots.Length - 1].MetaOffset + CurSlotSize;
            byte[] mapHalf = br.ReadBytes((int)(br.BaseStream.Length - br.BaseStream.Position));

            bw.BaseStream.Position = Slots[Slots.Length - 1].MetaOffset + CurSlotSize;
            bw.Write(dupArray);
            bw.Write(mapHalf);

            bw.BaseStream.Position = Map.tags.tagsList[0].MetaOffset + CurReflexiveInMeta;
            bw.Write(SlotsCount += 1);

            int shift = dupArray.Length;

            if (MapFilter == MapFilter.SinglePlayer) //have to fix mp reflex
            {
                br.BaseStream.Position = Map.tags.tagsList[0].MetaOffset + 15820 + 4;

                int mpOffset = br.ReadInt32() + shift;
                bw.BaseStream.Position -= 4;
                bw.Write(mpOffset);
            }

            /*
            //--[Reflexives]
            for (int y = 1; y < map.index.objectCount; y++)
            {
                //these tags are way above the norm meta
                if (map.tags.tagsList[y].Type == "sbsp" || map.tags.tagsList[y].Type == "ltmp") continue;

                //goto new metaoffset of tag + 4 (skip chunkcnt if ther is one)
                br.BaseStream.Position = (map.tags.tagsList[y].MetaOffset + shift) + 4;

                for (int x = 4; x < map.tags.tagsList[y].MetaSize; x += 4)
                {
                    int rawReflx = br.ReadInt32();
                    int transReflx = rawReflx - map.index.secondaryMagic;

                    //if the offset is within the current tags meta
                    if (transReflx >= map.tags.tagsList[y].MetaOffset &&
                        transReflx <= map.tags.tagsList[y].MetaOffset + map.tags.tagsList[y].MetaSize)
                    {
                        //we have a reflexive
                        bw.BaseStream.Position -= 4;
                        bw.Write(rawReflx + shift);

                        bw.BaseStream.Position += 4;
                        x += 4;
                    }
                }
            }
            */

            //--[Meta Offsets]
            for (int x = 1; x < Map.tags.tagsList.Length; x++)
            {
                if (Map.tags.tagsList[x].Type != "sbsp" && Map.tags.tagsList[x].Type != "ltmp")
                    Map.tags.tagsList[x].MetaOffset += shift;
            }

            //--[Tags]
            bw.BaseStream.Position = Map.index.offsetOfObjectIndex + 12;

            bw.Write(Map.tags.tagsList[0].MetaSize += shift);
            bw.BaseStream.Position += 8;

            for (int x = 1; x < Map.index.objectCount; x++)
            {
                if (Map.tags.tagsList[x].Type == "sbsp" || Map.tags.tagsList[x].Type == "ltmp") bw.BaseStream.Position += 16;
                else
                {
                    bw.Write(Map.tags.tagsList[x].RawMetaOffset += shift);
                    bw.BaseStream.Position += 12;
                }
            }

            //--[End]
            bw.BaseStream.Position = Map.tags.tagsList[Map.tags.tagsList.Length - 1].MetaOffset +
                Map.tags.tagsList[Map.tags.tagsList.Length - 1].MetaSize;

            int newPaddingLen = 512 - ((int)bw.BaseStream.Position % 512);
            newPaddingLen += newPaddingLen == 512 ? 2560 : 3072;

            int eof = (int)bw.BaseStream.Position + newPaddingLen;

            while (bw.BaseStream.Position < eof)
                bw.Write((byte)205); //CD

            bw.BaseStream.SetLength(eof);

            //--[Header]
            bw.BaseStream.Position = 8;
            bw.Write(eof);

            br.BaseStream.Position = 24;
            //int diff = (eof - map.header.metaStart) - map.header.metaSize;
            bw.Write(Map.header.metaSize += dupArray.Length);
            bw.Write(Map.header.nonRawSize += dupArray.Length);

            br.Close();
            bw.Close();
        }

        /// <summary>
        /// W.I.P
        /// </summary>
        /// <param name="slotIndex"></param>
        public void DeleteMapSlot(int slotIndex)
        {
            FileStream fs = new FileStream(Map.path, FileMode.Open,
                FileAccess.ReadWrite, FileShare.ReadWrite);

            BinaryReader br = new BinaryReader(fs);
            BinaryWriter bw = new BinaryWriter(fs);

            //sp and mp have diff sizes
            int shift = -CurSlotSize;

            br.BaseStream.Position = Slots[slotIndex].MetaOffset + CurSlotSize;
            byte[] mapHalf = br.ReadBytes((int)(br.BaseStream.Length - br.BaseStream.Position));

            bw.BaseStream.Position = Slots[slotIndex].MetaOffset;
            bw.Write(mapHalf);

            bw.BaseStream.Position = Map.tags.tagsList[0].MetaOffset + CurReflexiveInMeta;
            bw.Write(SlotsCount -= 1);

            if (MapFilter == MapFilter.SinglePlayer) //have to fix mp reflex
            {
                br.BaseStream.Position = Map.tags.tagsList[0].MetaOffset + 15820 + 4;

                int mpOffset = br.ReadInt32() + shift;
                bw.BaseStream.Position -= 4;
                bw.Write(mpOffset);
            }

            //--[Reflexives]
            for (int y = 1; y < Map.index.objectCount; y++)
            {
                //these tags are way above the norm meta
                if (Map.tags.tagsList[y].Type == "sbsp" || Map.tags.tagsList[y].Type == "ltmp") continue;

                //goto new metaoffset of tag + 4 (skip chunkcnt if ther is one)
                br.BaseStream.Position = (Map.tags.tagsList[y].MetaOffset + shift) + 4;

                for (int x = 4; x < Map.tags.tagsList[y].MetaSize; x += 4)
                {
                    int rawReflx = br.ReadInt32();
                    int transReflx = rawReflx - Map.index.secondaryMagic;

                    //if the offset is within the current tags meta
                    if (transReflx >= Map.tags.tagsList[y].MetaOffset &&
                        transReflx <= Map.tags.tagsList[y].MetaOffset + Map.tags.tagsList[y].MetaSize)
                    {
                        //we have a reflexive
                        bw.BaseStream.Position -= 4;
                        bw.Write(rawReflx + shift);

                        bw.BaseStream.Position += 4;
                        x += 4;
                    }
                }
            }

            //--[Meta Offsets]
            for (int x = 1; x < Map.tags.tagsList.Length; x++)
            {
                if (Map.tags.tagsList[x].Type != "sbsp" && Map.tags.tagsList[x].Type != "ltmp")
                    Map.tags.tagsList[x].MetaOffset += shift;
            }

            //--[Tags]
            bw.BaseStream.Position = Map.index.offsetOfObjectIndex + 12;

            bw.Write(Map.tags.tagsList[0].MetaSize += shift);
            bw.BaseStream.Position += 8;

            for (int x = 1; x < Map.index.objectCount; x++)
            {
                if (Map.tags.tagsList[x].Type == "sbsp" || Map.tags.tagsList[x].Type == "ltmp") bw.BaseStream.Position += 16;
                else
                {
                    bw.Write(Map.tags.tagsList[x].RawMetaOffset += shift);
                    bw.BaseStream.Position += 12;
                }
            }

            //--[End]
            bw.BaseStream.Position = Map.tags.tagsList[Map.tags.tagsList.Length - 1].MetaOffset +
                Map.tags.tagsList[Map.tags.tagsList.Length - 1].MetaSize;

            int newPaddingLen = 512 - ((int)bw.BaseStream.Position % 512);
            newPaddingLen += newPaddingLen == 512 ? 2560 : 3072;

            int eof = (int)bw.BaseStream.Position + newPaddingLen;

            while (bw.BaseStream.Position < eof)
                bw.Write((byte)205); //CD

            bw.BaseStream.SetLength(eof);

            //--[Header]
            bw.BaseStream.Position = 8;
            bw.Write(eof);

            br.BaseStream.Position = 24;
            int diff = (eof - Map.header.metaStart) - Map.header.metaSize;
            bw.Write(Map.header.metaSize += diff);
            bw.Write(Map.header.nonRawSize += diff);

            br.Close();
            bw.Close();
        }
    }

    public enum MapFilter
    {
        SinglePlayer,
        MultiPlayer
    }

    public struct MapSlot
    {
        public long MetaOffset;

        public int StartValue;
        public byte[] bitmTag;
        public int TagID;

        public string[] Names;
        public string internalName;

        public string[] Descriptions;
        public string internalDescription;

        public string Scenario;
        public int EndValue;
        public int Locked;
        public int TeamOptions;

        public BitmapInfo bInfo;
        public int ImgOffset;
        public long ImgSizePointer;
        public int ImgSize;

        public byte[] ImgBuffer;
        public Bitmap Image;
    }
}